@@ -35,6 +35,7 @@ from paramtools.select import (
     select_lt,
     select_lte,
 )
+from paramtools.sorted_key_list import SortedKeyList, SortedKeyListResult
 from paramtools.typing import ValueObject
 from paramtools.utils import (
     read_json,
@@ -47,12 +48,12 @@ from paramtools.utils import (
     hashable_value_object,
     filter_labels,
     make_label_str,
-    SortedKeyList,
 )
+from paramtools.values import Values, Slice, QueryResult
 
 
 name = "paramtools"
-__version__ = "0.15.1"
+__version__ = "0.17.0"
 
 __all__ = [
     "SchemaFactory",
@@ -97,5 +98,9 @@ __all__ = [
     "filter_labels",
     "make_label_str",
     "SortedKeyList",
+    "SortedKeyListResult",
     "ValueObject",
+    "Values",
+    "Slice",
+    "QueryResult",
 ]
